import java.net.URL

/**
 * 系统初始化脚本
 */
import groovy.ui.SystemOutputInterceptor

import com.kdayun.manager.entity.CoreTables
import com.kdayun.manager.service.CoreTablesService
import com.kdayun.manager.service.CoreDbService
import com.kdayun.manager.service.impl.CoreTablesServiceImpl
import com.kdayun.z1.core.context.Context
import com.kdayun.z1.core.message.MessageUtil
import com.kdayun.z1.core.util.FileUitls
import com.kdayun.z1.core.util.serialize.Serializer
import com.kdayun.z1.core.util.JarUtils;


CORE_TABLES_LIST=["CORE_TABLES", "CORE_TABLES_COLUMNS", "CORE_TABLES_INDEXS", "CORE_TABLES_LINKS","CORE_SYSLOG"];
MSG_FORMAT="{'msg':'%s'}";
PROCESS_FORMAT="{'process':%d , 'total':%d}";
/**
 * 导入模型文件
 * @author gk @date 2018年9月27日 下午4:04:27
 * @param file
 * @throws Exception
 */
public void ImportModelFile(URL file) throws Exception {
	CoreDbService coreDbService=Context.getInstance().getBean(CoreDbService.class);
	Serializer gs = coreDbService.getNewModelSerializer();

	CoreTables model = (CoreTables) gs.Deserialize(
			JarUtil.readFileContext(file.getInputStream(), CoreTables.class);
	String msg=String.format(MSG_FORMAT,"处理【"+model.getMX_BIANH()+ "】模型...");
	Context.getInstance().messageService.sendMessage(MessageUtil.WM_INIT_PROCESS, msg);
	/**TO DO 初始化才需要运行**/
	if (coreDbService.isTableExists(model))
		coreDbService.dropTable(model);
	//创建平台表
	if (CORE_TABLES_LIST.contains(model.getMX_BIANH())) {
		//创建表结构
		coreDbService.createTableFromJsonModel(model);
	} else {
		//创建表结构
		coreDbService.createTableFromJsonModel(model);
		//插入数据
		coreDbService.insertRowFromJsonModelRows(model);
		//插入表模型
		coreDbService.insertModelFromJsonModel(model);
	}
}

/**
 * 脚本的入口
 * @author gk @date 2018年9月29日 下午1:54:22
 */
public void start() {
	System.out.println("平台初始化脚本开始运行...");
	Context.getInstance().messageService.sendMessage(MessageUtil.WM_INIT_PROCESS, String.format(MSG_FORMAT,"平台初始化脚本开始运行...",0));
	CoreTablesServiceImpl service=Context.getInstance().getBean(CoreTablesService.class);
	CoreDbService coreDbService=Context.getInstance().getBean(CoreDbService.class);
	coreDbService.setInitSystem(true);
	try{
		URL fileUrl = Thread.currentThread().getContextClassLoader().getResource(INIT_MAIN_FILE);
		List files = JarUtil.getFiles(dir, "^init/.*\\.json$");
		System.out.println("平台初始化脚本开始运行,需要初始化的文件数量【"+files.size()+"】 ");
		Context.getInstance().messageService.sendMessage(MessageUtil.WM_INIT_PROCESS, String.format(MSG_FORMAT,"需要初始化的文件数量【"+files.size()+"】"));
		//排序
		files.sort{a,b->
			String o1Name = FileUitls.extractFileNameWithNoExt(((File) a).getPath());
			String o2Name = FileUitls.extractFileNameWithNoExt(((File) b).getPath());
			if (CORE_TABLES_LIST.contains(o1Name) && CORE_TABLES_LIST.contains(o2Name)) {
				return CORE_TABLES_LIST.indexOf(o1Name) < CORE_TABLES_LIST.indexOf(o2Name) ? -1 : 1;
			}
			if (CORE_TABLES_LIST.contains(o1Name)) {
				return -1;
			}
			if (CORE_TABLES_LIST.contains(o2Name)) {
				return 1;
			}
			return o1Name.compareTo(o2Name);
		}
		Integer index=0;
		for (URL file : files) {
			index++
			Context.getInstance().messageService.sendMessage(MessageUtil.WM_INIT_PROCESS, String.format(PROCESS_FORMAT,index,files.size()));
			String ModelName=FileUitls.extractFileNameWithNoExt(file.getPath());
			if(modelList==null ){
				ImportModelFile(file);
			}else if (modelList.contains(ModelName)){
				ImportModelFile(file);
			}
		}
	}
	finally{
		coreDbService.setInitSystem(false);
	}
	Context.getInstance().messageService.sendMessage(MessageUtil.WM_INIT_PROCESS,  String.format(MSG_FORMAT,"平台初始化脚本运行结束..."));
	System.out.println("平台初始化脚本运行结束...");
}
start();